@@ -15,70 +15,6 @@
         av_push(export, sv); \
     }
 
-static void
-Cache_LibMemcached_uncompress(cache, value, value_len, flags)
-        Cache_LibMemcached *cache;
-        char **value;
-        size_t *value_len;
-        uint16_t flags;
-{
-    if (MEMCACHED_HAVE_ZLIB(cache) && (flags & F_COMPRESS)) {
-        SV *sv;
-        dSP;
-
-        ENTER;
-        SAVETMPS;
-
-        PUSHMARK(SP);
-        EXTEND(SP, 1);
-        PUSHs(sv_2mortal(newSVpv(*value, *value_len)));
-        PUTBACK;
-
-        if (call_pv("Compress::Zlib::memGunzip", G_SCALAR) <= 0) {
-            croak("Compress::Zlib::memGunzip did not return a proper value");
-        }
-        SPAGAIN;
-
-        sv = POPs;
-        *value = SvPV(sv, *value_len);
-
-        FREETMPS;
-        LEAVE;
-    }
-}
-
-static void
-Cache_LibMemcached_thaw(cache, sv, flags)
-        Cache_LibMemcached *cache;
-        SV *sv;
-        uint16_t flags;
-{
-    if (flags & F_STORABLE) {
-        SV *rv;
-        dSP;
-        ENTER;
-        SAVETMPS;
-
-        PUSHMARK(SP);
-        EXTEND(SP, 1);
-        PUSHs(sv);
-        PUTBACK;
-
-        if (call_pv("Storable::thaw", G_SCALAR) <= 0) {
-            croak("Storable::thaw did not return a proper value");
-        }
-        SPAGAIN;
-
-        rv = POPs;
-        
-        SvSetSV(sv, rv);
-
-        FREETMPS;
-        LEAVE;
-    }
-}
-
-
 void
 Cache_LibMemcached_bootstrap()
 {
@@ -125,9 +61,235 @@ Cache_LibMemcached_bootstrap()
     CONSTSUBi( "MEMCACHED_MAXIMUM_RETURN", MEMCACHED_MAXIMUM_RETURN );
 }
 
+/* The next functions (compress, uncompress, freeze, thaw) are implemented
+ * as simple static method calls emulating a Perl method calls to
+ * Compress::Zlib and Storable, respectively
+ */
+
+inline static void
+Cache_LibMemcached_uncompress(cache, value, value_len, flags)
+        Cache_LibMemcached *cache;
+        char **value;
+        size_t *value_len;
+        uint16_t flags;
+{
+    if (MEMCACHED_HAVE_ZLIB(cache) && (flags & F_COMPRESS)) {
+        SV *sv;
+        dSP;
+
+        ENTER;
+        SAVETMPS;
+
+        PUSHMARK(SP);
+        EXTEND(SP, 1);
+        PUSHs(sv_2mortal(newSVpv(*value, *value_len)));
+        PUTBACK;
+
+        if (call_pv("Compress::Zlib::memGunzip", G_SCALAR) <= 0) {
+            croak("Compress::Zlib::memGunzip did not return a proper value");
+        }
+        SPAGAIN;
+
+        sv = POPs;
+        *value = SvPV(sv, *value_len);
+
+        FREETMPS;
+        LEAVE;
+    }
+}
+
+inline static void
+Cache_LibMemcached_thaw(cache, sv, flags)
+        Cache_LibMemcached *cache;
+        SV *sv;
+        uint16_t flags;
+{
+    if (flags & F_STORABLE) {
+        SV *rv;
+        dSP;
+        ENTER;
+        SAVETMPS;
+
+        PUSHMARK(SP);
+        EXTEND(SP, 1);
+        PUSHs(sv);
+        PUTBACK;
+
+        if (call_pv("Storable::thaw", G_SCALAR) <= 0) {
+            croak("Storable::thaw did not return a proper value");
+        }
+        SPAGAIN;
+
+        rv = POPs;
+        
+        SvSetSV(sv, rv);
+
+        FREETMPS;
+        LEAVE;
+    }
+}
+
+inline static void
+Cache_LibMemcached_freeze(cache, sv, flags)
+        Cache_LibMemcached *cache;
+        SV *sv;
+        uint16_t *flags;
+{
+    if (SvROK(sv)) {
+        SV *rv;
+        dSP;
+        ENTER;
+        SAVETMPS;
+
+        PUSHMARK(SP);
+        EXTEND(SP, 1);
+        PUSHs(sv);
+        PUTBACK;
+
+        if (call_pv("Storable::freeze", G_SCALAR) <= 0) {
+            croak("Storable::freeze did not return a proper value");
+        }
+        SPAGAIN;
+
+        rv = POPs;
+        
+        SvSetSV(sv, rv);
+
+        FREETMPS;
+        LEAVE;
+
+        *flags |= F_STORABLE;
+    }
+}
+
+inline static void
+Cache_LibMemcached_compress(cache, sv, flags)
+        Cache_LibMemcached *cache;
+        SV *sv;
+        uint16_t *flags;
+{
+    size_t compress_threshold;
+    bool   compress_enabled;
+    size_t sv_length = sv_len(sv);
+
+    compress_threshold = MEMCACHED_COMPRESS_THRESHOLD(cache);
+    if (MEMCACHED_HAVE_ZLIB(cache) && 
+        MEMCACHED_COMPRESS_ENABLED(cache) &&
+        compress_threshold > 0 &&
+        sv_length > compress_threshold
+    ) {
+        SV *rv;
+        size_t rv_length;
+        dSP;
+
+        ENTER;
+        SAVETMPS;
+
+        PUSHMARK(SP);
+        EXTEND(SP, 1);
+        PUSHs(sv);
+        PUTBACK;
+
+        if (call_pv("Compress::Zlib::memGzip", G_SCALAR) <= 0) {
+            croak("Compress::Zlib::memGzip did not return a proper value");
+        }
+        SPAGAIN;
+
+        rv = POPs;
+        rv_length = SvCUR(rv);
+
+        /* We won't reset the contents of the original SV unless
+         * the savings are actually greater than the savings threshold
+         */
+        if (rv_length < sv_length * ( 1 - MEMCACHED_COMPRESS_SAVINGS(cache) ) ) {
+            SvSetSV(sv, rv);
+            *flags |= F_COMPRESS;
+        }
+        FREETMPS;
+        LEAVE;
+
+    }
+}
+
+Cache_LibMemcached_rc
+Cache_LibMemcached_set(cache, key, value, expires)
+        Cache_LibMemcached *cache;
+        SV *key;
+        SV *value;
+        time_t expires;
+{
+    STRLEN key_len, value_len;
+    char *key_char;
+    char *value_char;
+    uint16_t flags = 0;
+    SV *value_copy;
+
+    value_copy = newSVsv(value);
+
+    Cache_LibMemcached_freeze(cache, value_copy, &flags);
+    Cache_LibMemcached_compress(cache, value_copy, &flags);
+
+    key_char = SvPVbyte(key, key_len);
+    value_char = SvPVbyte(value_copy, value_len);
+
+    return memcached_set(MEMCACHED_CACHE(cache), key_char, key_len, value_char, value_len, expires, flags);
+}
+
+Cache_LibMemcached_rc
+Cache_LibMemcached_add(cache, key, value, expires)
+        Cache_LibMemcached *cache;
+        SV *key;
+        SV *value;
+        time_t expires;
+{
+    STRLEN key_len, value_len;
+    char *key_char;
+    char *value_char;
+    uint16_t flags = 0;
+    SV *value_copy;
+
+    value_copy = newSVsv(value);
+
+    Cache_LibMemcached_freeze(cache, value_copy, &flags);
+    Cache_LibMemcached_compress(cache, value_copy, &flags);
+
+    key_char = SvPVbyte(key, key_len);
+    value_char = SvPVbyte(value_copy, value_len);
+
+    return memcached_add(MEMCACHED_CACHE(cache), key_char, key_len, value_char, value_len, expires, flags);
+}
+
+Cache_LibMemcached_rc
+Cache_LibMemcached_replace(cache, key, value, expires)
+        Cache_LibMemcached *cache;
+        SV *key;
+        SV *value;
+        time_t expires;
+{
+    STRLEN key_len, value_len;
+    char *key_char;
+    char *value_char;
+    uint16_t flags = 0;
+    SV *value_copy;
+
+    value_copy = newSVsv(value);
+
+    Cache_LibMemcached_freeze(cache, value_copy, &flags);
+    Cache_LibMemcached_compress(cache, value_copy, &flags);
+
+    key_char = SvPVbyte(key, key_len);
+    value_char = SvPVbyte(value_copy, value_len);
+
+    return memcached_set(MEMCACHED_CACHE(cache), key_char, key_len, value_char, value_len, expires, flags);
+}
+
 SV *
-Cache_LibMemcached_create(pkg, have_zlib)
+Cache_LibMemcached_create(pkg, have_zlib, compress_enabled, compress_threshold, compress_savings)
         char *pkg;
+        bool have_zlib;
+        bool compress_enabled;
+        size_t compress_threshold;
+        float compress_savings;
 {
     SV *sv;
     Cache_LibMemcached *xs;
@@ -135,9 +297,13 @@ Cache_LibMemcached_create(pkg, have_zlib)
 
     Newxz(xs, 1, Cache_LibMemcached);
 
-    MEMCACHED_CACHE(xs) =  memcached_create(NULL);
-    MEMCACHED_HAVE_ZLIB(xs) = have_zlib;
-    XS_STRUCT2OBJ(sv, "Cache::Memcached::LibMemcached::Backend", xs);
+    MEMCACHED_CACHE(xs)              = memcached_create(NULL);
+    MEMCACHED_HAVE_ZLIB(xs)          = have_zlib;
+    MEMCACHED_COMPRESS_ENABLED(xs)   = compress_enabled;
+    MEMCACHED_COMPRESS_THRESHOLD(xs) = compress_threshold;
+    MEMCACHED_COMPRESS_SAVINGS(xs)   = compress_savings;
+
+    XS_STRUCT2OBJ(sv, "Cache::Memcached::LibMemcached", xs);
 
     return sv;
 }
@@ -149,7 +315,7 @@ Cache_LibMemcached_DESTROY(cache)
     memcached_free(MEMCACHED_CACHE(cache));
 }
 
-memcached_return
+Cache_LibMemcached_rc
 Cache_LibMemcached_server_add(cache, hostname, port)
         Cache_LibMemcached *cache;
         char *hostname;
@@ -158,7 +324,7 @@ Cache_LibMemcached_server_add(cache, hostname, port)
     return memcached_server_add(MEMCACHED_CACHE(cache), hostname, port);
 }
 
-memcached_return
+Cache_LibMemcached_rc
 Cache_LibMemcached_server_add_unix_socket(cache, filename)
         Cache_LibMemcached *cache;
         char *filename;
@@ -166,22 +332,18 @@ Cache_LibMemcached_server_add_unix_socket(cache, filename)
     return memcached_server_add_unix_socket(MEMCACHED_CACHE(cache), filename);
 }
 
-memcached_return
-Cache_LibMemcached_set_raw(cache, key, value, expires, flags)
+unsigned int
+Cache_LibMemcached_server_list_count(cache)
         Cache_LibMemcached *cache;
-        SV *key;
-        SV *value;
-        time_t expires;
-        unsigned int flags;
 {
-    STRLEN key_len, value_len;
-    char *key_char;
-    char *value_char;
+    return memcached_server_list_count( MEMCACHED_CACHE(cache)->hosts );
+}
 
-    key_char = SvPVbyte(key, key_len);
-    value_char = SvPVbyte(value, value_len);
-
-    return memcached_set(MEMCACHED_CACHE(cache), key_char, key_len, value_char, value_len, expires, flags);
+void
+Cache_LibMemcached_server_list_free(cache)
+        Cache_LibMemcached *cache;
+{
+    memcached_server_list_free( MEMCACHED_CACHE(cache)->hosts );
 }
 
 SV *
@@ -231,10 +393,10 @@ Cache_LibMemcached_mget(cache, keys, key_len_list, keys_len)
     hv = newHV();
     while (1) {
         char *value;
-        char *key;
-        size_t key_length;
-        size_t value_length;
-        uint16_t flags;
+        char key[MEMCACHED_MAX_KEY];
+        size_t key_length = 0;
+        size_t value_length = 0;
+        uint16_t flags = 0;
         Cache_LibMemcached_rc rc;
         SV *sv;
 
@@ -320,4 +482,12 @@ Cache_LibMemcached_decr(cache, key_sv, offset)
     return newSViv(value);
 }
 
+Cache_LibMemcached_rc
+Cache_LibMemcached_flush_all(cache, expiration) 
+        Cache_LibMemcached *cache;
+        time_t expiration;
+{
+    return memcached_flush( MEMCACHED_CACHE(cache), expiration );
+}
+
 #endif /* __PERL_LIBMEMCACHED_C__ */
